****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_aggregate_tables.sas  
*
* Created (mm/dd/yyyy): 12/18/2018
* Last modified: 02/05/2019
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: The macro imports and aggregates across runid the following tables relevant to a Type 5 report:
*			-[runid]_baseline_[periodid]
*			-[runid]_t5_cida_disp_by_daysupp
*			-[runid]_t5_cida_episdur
*			-[runid]_t5_cida_episdur_censor
*			-[runid]_t5_cida_gaps
*			-[runid]_t5_cida_firsteps
*			-[runid]_attrition	
*
*  Program inputs:                                                                                   
*  	-
* 
*  Program outputs:                                                                                                                                       
*  	-
* 
*  PARAMETERS:        

*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  02/05/19	  AP		Added Figure 5 to censor table aggregation
*
***************************************************************************************************;

%macro ms_t5_aggregate_tables();

	%put =====> MACRO CALLED: ms_t5_aggregate_tables v1.1;

	/******************/
	/* Baseline table */
	/******************/
	%if "&output_baselinetable" = "Y" %then %do;

	/*Loop through groups in the sort order*/
		%do order = 1 %to &num_allgroups.;

		 	data _null_;
				set &groupfile.(where=(order=&order.));
				call symputx("runid", runid);
				call symputx("group" , group);
			run;
	
		 	%do i = %eval(&look_start) %to %eval(&look_end);

				%let infile = &runid._baseline_&i; 
			 	%if %sysfunc(exist(msoc.&infile.))=0 %then %do;
			 		%put WARNING: &infile. does not exist. No exposures in &runid. Program will abort;
				 	%abort; 
			 	%end;
			 	%else %do;
	 
					data temp_grp&order._&i.; 
						set msoc.&infile.; 
						where lowcase(group) = "&group.";
					run;

					/*Transpose*/
					proc transpose data=temp_grp&order._&i. out=table1_grp&order._&i.; 
						id group;
					run;
						
					proc sort data =table1_grp&order._&i.; 
						by _name_; 
					run;

					*if no covars need to initialize _label_;
					data table1_grp&order._&i.; 
						set table1_grp&order._&i.;
							if _n_ = 1 then do;
								dsid = open("table1_grp&order._&i.");
								if varnum(dsid,"_label_") = 0 then do;
									format _label_ $70.;
									_label_ ='';
								end;
								rc= close(dsid);
							end;
						drop rc dsid;
					run;	
				%end;
			%end;
		%end;
	%end;

	/****************************/
	/* t5_cida_disp_by_daysupp  */
	/* t5_cida_episdur 			*/
	/* t5_cida_episdur_censor 	*/
	/* t5_cida_gaps 			*/
	/* t5_cida_firsteps			*/
	/****************************/

	%macro agg_t5(infile=, outfile=);
		%do r = 1 %to %sysfunc(countw(&runidlist.));

		   %if %sysfunc(exist(MSOC.&&runid&r.._&infile.))=0 %then %do;
			   %put WARNING: &&runid&r.._&infile. does not exist. Program will abort;
			   %abort; 
		   %end;
		   %else %do;
				data &infile.; 
					set msoc.&&runid&r.._&infile.; 
					where lowcase(group) in (&&grouplist&r.); 
				  	group=lowcase(group);
					format runid $10.;
					runid = lowcase("&&runid&r.");
			   	run;
								
				/* Aggregate Data */
			   	%if &r=1 %then %do;
					data &outfile.; 
						set &infile.; 
					run;
			   	%end;
			   	%else %do;
					proc append data=&infile. base=&outfile. force; run;
			   	%end;

			   proc datasets nowarn noprint nolist lib=work; delete &infile.; quit;	
		   %end;	
		%end;
	%mend agg_t5;

	/*t5_cida_episdur*/
	%if %sysfunc(prxmatch(m/T2|T3|T4|T4|T5|T6|T7|T8|T9/i,&tablelist.)) > 0 %then %do;
		%agg_t5(infile=t5_cida_episdur, outfile=agg_episdur);
	%end;

	/*t5_cida_disp_by_daysupp*/
	%if %sysfunc(prxmatch(m/T10|T11/i,&tablelist.)) > 0 %then %do;
	  %agg_t5(infile=t5_cida_disp_by_daysupp, outfile=agg_disp);
	%end;

	/*t5_cida_gaps*/
	%if %sysfunc(prxmatch(m/T12|T13|T14/i,&tablelist.)) > 0 %then %do;
	  %agg_t5(infile=t5_cida_gaps, outfile=agg_gaps);
	%end;

	/*t5_cida_episdur_censor*/
	%if %sysfunc(prxmatch(m/T15|F4|F5/i,&tablelist.)) > 0 %then %do;
	  %agg_t5(infile=t5_cida_episdur_censor, outfile=agg_episdur_censor);
	%end;
	
	/*t5_cida_firsteps*/
	%if %sysfunc(prxmatch(m/F1|F2|F3/i,&tablelist.)) > 0 %then %do;
	  %agg_t5(infile=t5_cida_firsteps, outfile=agg_firsteps);
	%end;

	/*******************/
	/* Attrition table */
	/*******************/
	%agg_t5(infile=attrition, outfile=agg_attr);	

	%put =====> END MACRO: ms_t5_aggregate_tables v1.1;

%mend ms_t5_aggregate_tables;
