****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_createreport.sas  
*
* Created (mm/dd/yyyy): 12/18/2018
* Last modified: 02/06/2019
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This creates aggregated tables to produce a Type 5 report.
*   
*  Program inputs:                                                                                   
* 
*  Program outputs:                                                                                
* 
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  2/6/2019	 AP			Added Figure 5
*
***************************************************************************************************;

%macro ms_t5_createreport();

/**************************************************************************************************
*	1. Assign default macros vars															
***************************************************************************************************/

	%put =====> MACRO CALLED: ms_t5_createreport v1.1;

	/* set up parameters for groups */
	%if %sysfunc(exist(infolder.&groupfile. )) = 0 %then %do;
		%put WARNING: GroupFile does not exist. Confirm file and path are specified correctly;
		%abort;
	%end;

	data &groupfile.;
		set infolder.&groupfile.;
		runid = lowcase(runid);
		group = lowcase(group);
		output_figure5=upcase(output_figure5);
	run;

	/* check parameters for tables */
	%if %sysfunc(exist(infolder.&tablefile.)) = 0 %then %do;
	    %put WARNING: table file does not exist. Confirm file and path are specified correctly;
		%abort;
	%end;

	*Read in tablefile, add defensive coding, and merge in levelid lookup table with userstrata table;
	data &tablefile.;
		set infolder.&tablefile.;
		table=upcase(table);
		tablesub=upcase(tablesub);
	run;

	data &levellookup.(rename=levelid1_out=levelid1 rename=levelid2_out=levelid2);
		set infolder.&levellookup.;
		table=upcase(table);
		tablesub=upcase(tablesub);
		levelid1 = lowcase(levelid1);
		levelid2 = lowcase(levelid2);
		dataset = lowcase(dataset);

		*defensive: replace overall with missing;
		if levelid1 = 'overall' then levelid1 = '';
		if levelid2 = 'overall' then levelid2 = '';

		*alphabetize levelid vars;
	   	array t[10] $50 _temporary_;
	  	call missing(of t[*]);
	   	do i = 1 to dim(t) until(p eq 0);
	    	call scan(levelid1,i,p,l);
	      	t[i] = substrn(levelid1,p,l);
	    end;
	    call sortc(of t[*]);
	    length levelid1_out $50;
	    levelid1_out = catx(' ',of t[*]);
	    drop i p l;

  	    array b[10] $50 _temporary_;
	    call missing(of b[*]);
	    do i = 1 to dim(b) until(p eq 0);
	        call scan(levelid2,i,p,l);
	    	b[i] = substrn(levelid2,p,l);
	    end;
	    call sortc(of b[*]);
	    length levelid2_out $50;
	    levelid2_out = catx(' ',of b[*]);
	    drop i p l levelid1 levelid2;
	run;

	data &userstrata.(rename=levelvars_out=levelvars);
   		set infolder.&userstrata.;
		tableid = lowcase(tableid);
		levelvars=lowcase(levelvars);

		*alphabetize levelid vars;
	   	array t[10] $50 _temporary_;
	  	call missing(of t[*]);
	   	do i = 1 to dim(t) until(p eq 0);
	    	call scan(levelvars,i,p,l);
	      	t[i] = substrn(levelvars,p,l);
	    end;
	    call sortc(of t[*]);
	    length levelvars_out $50;
	    levelvars_out = catx(' ',of t[*]);
	    drop i p l levelvars;
	run;

	*Merge in levelids - need to do twice, 1 for each levelid;
	proc sql noprint undo_policy=none;
		create table &tablefile. as
		select distinct table.table
			 , table.tablesub
			 , table.disptablenum
			 , lkup.dataset
			 , lkup.levelid1 as strat1
			 , lkup.levelid2 as strat2
			 , strata.levelid as levelid1
		from &tablefile. as table
		left join &levellookup. as lkup
		on table.table = lkup.table and table.tablesub=lkup.tablesub
		left join &userstrata as strata
		on strata.tableid = lkup.dataset and strata.levelvars = lkup.levelid1;
	quit;

	proc sql noprint undo_policy=none;
		create table &tablefile. as
		select distinct table.*
			 , strata.levelid as levelid2
		from &tablefile. as table
		left join &levellookup. as lkup
		on table.table = lkup.table and table.tablesub=lkup.tablesub
		left join &userstrata as strata
		on strata.tableid = lkup.dataset and strata.levelvars = lkup.levelid2;
	quit;

	*Defensive check - if levels missing, write warning to the log and abort;
	data levelid1_check;
		set &tablefile.(where=(levelid1 is missing and Table ne 'T16'));
	run;

	%isdata(dataset=levelid1_check);
	%if %eval(&nobs.>0) %then %do;
		proc sql noprint;
			select table, tablesub into :levelid1_tablecheck1 - :levelid1_tablecheck&nobs., :levelid1_subcheck1 - :levelid1_subcheck&nobs.
			from levelid1_check;
		quit;

		%do n=1 %to &nobs.;
			%let levelid1_tablecheck&n = &&levelid1_tablecheck&n.;
			%let levelid1_subcheck&n. = &&levelid1_subcheck&n.;
			%put ERROR: Unable to generate the following report tables and stratifications:&&levelid1_tablecheck&n..:&&levelid1_subcheck&n..;
		%end;
		%put Check USERSTRATA or TABLEFILE. Program will abort;
		%abort;
	%end;

	data levelid2_check;
		set &tablefile.(where=(levelid2 is missing and table in ('T2','T3','T4','T5','T6','T7','T8','T9','T10','T11')));
	run;

	%isdata(dataset=levelid2_check);
	%if %eval(&nobs.>0) %then %do;
		proc sql noprint;
			select table, tablesub into :levelid2_tablecheck1 - :levelid2_tablecheck&nobs., :levelid2_subcheck1 - :levelid2_subcheck&nobs.
			from levelid2_check;
		quit;

		%do n=1 %to &nobs.;
			%let levelid2_tablecheck&n = &&levelid2_tablecheck&n.;
			%let levelid2_subcheck&n. = &&levelid2_subcheck&n.;
			%put ERROR: Unable to generate the following report tables and stratfications: &&levelid2_tablecheck&n..:&&levelid2_subcheck&n..;
		%end;
		%put Check USERSTRATA or TABLEFILE. Program will abort;
		%abort;
	%end;

	/* check parameters for groups */
	%ms_t5_set_reportparameters;
	
/************************************************************************************************************************************
*	2. Import and aggregate data														
************************************************************************************************************************************/

	%ms_t5_aggregate_tables();

/************************************************************************************************************************************
*	3. Compute tables													
************************************************************************************************************************************/

	/**************************************************************************************************
	* Baseline Table													
	***************************************************************************************************/
	%if "&output_baselinetable" = "Y" %then %do;
		/*loop through each look*/
		%do i = %eval(&look_start) %to %eval(&look_end);
		    %ms_compute_baselinetable();
		%end; *loop through periodid;
	%end; *produce baseline table;

	/**************************************************************************************************
	* Distribution and Censor Counts tables												
 	***************************************************************************************************/
	*** Distribution of total episode duration ***;
	%if %sysfunc(prxmatch(m/T2|T3/i,&tablelist.)) > 0 %then %do;
		%ms_t5_create_distribution_tables(data=agg_episdur, 
									   	 countvar=npts,
									     catvar=cumepisodelength, 
									     categories =&totepisdurstrat.,
									     cattable = T2,
									     disttable =T3);
	%end;

	*** Distribution of episode duration ***;
	data _chk_table;
		set &tablefile.(where=(table in ('T4', 'T5', 'T6','T7', 'T8', 'T9')));
	run;
	%isdata(dataset=_chk_table);

	%if &nobs>0 %then %do;
		proc sql noprint;
			select distinct levelid1, levelid2 into: levelist1 separated by " ", :levelist2 separated by " "
			from _chk_table;
		quit;

		%create_comma_charlist(inlist=&levelist1, outlist=levelist1n);
		%create_comma_charlist(inlist=&levelist2, outlist=levelist2n);

		%put &=levelist1n &=levelist2n;

		data agg_episdur_first agg_episdur_secondpl agg_episdur_all;
		    set agg_episdur;
			if  level in (&levelist1n., &levelist2n.) and episodenum <=1 then output agg_episdur_first;
	        if  level in (&levelist1n., &levelist2n.) and episodenum >=2 then output agg_episdur_secondpl;
	        if  level in (&levelist1n., &levelist2n.) then output agg_episdur_all;
	    run;

		proc contents data=agg_episdur out=agg_episdur_outnames(keep=name) noprint;
		run;
		proc sql noprint;
			select distinct name into: classvarlist separated by ' '
			from agg_episdur_outnames
			where lowcase(name) ne 'npts' & index(lowcase(name), 'episode') = 0;
		quit;
		%put &classvarlist;
		
		/*** First episode duration ***/
		%if %sysfunc(prxmatch(m/T4|T5/i,&tablelist.)) > 0 %then %do;

			proc means data=agg_episdur_first noprint nway;
				var episodes;
				class &classvarlist. episodelength / missing;
				output out=agg_episdur_first(drop=_:) sum=;
			run;

			%ms_t5_create_distribution_tables(data=agg_episdur_first, 
										   	 countvar=episodes,
										     catvar=episodelength, 
										     categories =&episdurstrat.,
										     cattable = T4,
										     disttable =T5);
		%end;

		/*** Second episode duration ***/
		%if %sysfunc(prxmatch(m/T6|T7/i,&tablelist.)) > 0 %then %do;

		    proc means data=agg_episdur_secondpl noprint nway;
				var episodes;
				class &classvarlist. episodelength  / missing;
				output out=agg_episdur_secondpl(drop=_:) sum=;
			run;

			%ms_t5_create_distribution_tables(data=agg_episdur_secondpl, 
										   	 countvar=episodes,
										     catvar=episodelength, 
										     categories =&episdurstrat.,
										     cattable = T6,
										     disttable =T7);
		%end;
				     
		/*** All episode duration ***/
		%if %sysfunc(prxmatch(m/T8|T9/i,&tablelist.)) > 0 %then %do;

			proc means data=agg_episdur_all noprint nway;
				var episodes;
				class &classvarlist. episodelength  / missing;
				output out=agg_episdur_all(drop=_:) sum=;
			run;

			%ms_t5_create_distribution_tables(data=agg_episdur_all, 
										   	 countvar=episodes,
										     catvar=episodelength, 
										     categories =&episdurstrat.,
										     cattable = T8,
										     disttable =T9);
		%end;
	%end;

    *** Distribution of days supplied per dispensing (using AdjustedCodeCount) ***;
	%if %sysfunc(prxmatch(m/T10|T11/i,&tablelist.)) > 0 %then %do;
		%ms_t5_create_distribution_tables(data=agg_disp, 
									   	 countvar=adjustedcodecount,
									     catvar=daysupp, 
									     categories =&dispdayssupstrat.,
									     cattable = T10,
									     disttable =T11);
	%end;

	*** Descriptive statistics for gaps between treatment episodes ***;
	%if %sysfunc(prxmatch(m/T12|T13|T14/i,&tablelist.)) > 0 %then %do;
	    %ms_t5_create_gaps_table();
	%end;
		   
	/*** Counts fo reason for censoring ***/
	%if %sysfunc(prxmatch(m/T15/i,&tablelist.)) > 0 %then %do;
	    %ms_t5_create_censoring_table();
	%end;
			    
 	/**************************************************************************************************
	* Attrition Table											
	***************************************************************************************************/

	%if %sysfunc(prxmatch(m/T16/i,&tablelist.)) > 0 %then %do;

		/*Aggregate attrition table*/
	    proc sql;
	    create table agg_attr1 as
		select a.*, b.exposure, b.order
		from agg_attr as a,
	         &groupfile. as b
		where a.group=b.group;
		quit;

		proc means data=agg_attr1 noprint nway;
			var remaining excluded;
			class order exposure level descr;
			output out=agg_attr(drop=_:) sum=;
			where missing(descr) = 0 and substr(descr,1,11) ne 'Information'; 
		run;

		*put total number of patients in macro variable;
		proc sql noprint;
			select put(remaining, 12.0) into: totpat
			from agg_attr
			where level = 1;
		quit;

		data Attrition(rename=(descr1=descr));
			set agg_attr;
			format percent $22.;
			       percent =  cat(strip(put(remaining, comma12.0)),' (',strip(put((remaining/&totpat.),percent8.1)),')');
			descr1=tranwrd(descr,'96'x,'2D'x);
			drop descr;
		run;

	%end;

	/**************************************************************************************************
	* Figures												
	***************************************************************************************************/

	%if %sysfunc(prxmatch(m/F1|F2|F3|F4|F5/i,&tablelist.)) > 0 %then %do;
		%ms_t5_create_figures();
	%end;

	/**************************************************************************************************
	 * Start and End dates for each DP											
	 ***************************************************************************************************/
    data dpdates;
		maskedID = "DP01";
	    querymax =input("&enddate.", mmddyy10.);
	    dpmin = input(strip("&DP_MinDate."), 8.);
	    dpmax = input(strip("&DP_MaxDate."), 8.);
	    maxdate = min(querymax, dpmax);
	         
	    format dpmin maxdate mmddyy10.;
	    keep maskedID dpmin maxdate;
		output;
	run;

	/**************************************************************************************************
	* Table of contents												
	***************************************************************************************************/

    %ms_t5_create_toc();

/**************************************************************************************************
*	4. Create Report											
***************************************************************************************************/

     %ms_t5_output_report();

/**************************************************************************************************
*	5. Finish up										
***************************************************************************************************/

     proc datasets nowarn nolist lib=work kill; quit;

    %put NOTE: ******** END OF MACRO: ms_t5_createreport v1.1 ********;

	/* End log */
	proc printto;
	run;

%mend ms_t5_createreport;
