****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_concomitantusetables.sas  
*
* Created (mm/dd/yyyy): 06/04/2018
* Last modified:
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the output tables for concomitant episode cohorts.                                        
*   
*  Program inputs:    
*                                     
*  Program outputs:  
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_concomitantusetables();

%put =====> MACRO CALLED: ms_concomitantusetables v1.0;

	*Select all output levels relevant to t1_cida/t2_cida;
	data _t2conclevels;
		set userstrata;
		where lowcase(tableID) = "t2conc";
	run;

	%isdata(dataset=_t2conclevels);
	%if %eval(&nobs.>0) %then %do;

		** Loop through analysis groups ** ;
		%isdata(dataset=infolder.&concfile.); 
		data _null_;
	    	call symputx("num_order","&nobs.");
	    run;

		%do group=1 %to &num_order.;

			data _null_;
				set infolder.&concfile.;
				if _n_ = &group. then do;
					call symputx("analysisgrp", analysisgrp);
					call symputx("primary", lowcase(primary));
				end;
			run;
		
			data _null_;
				set COHORTFILE;
				if lowcase(cohortgrp) = "&primary." then do;
					*Age stratification;
					if missing(agestrat) then do;
						call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
					end;
					else do;
						call symputx("agestrat",agestrat);
					end;
				end;
			run;

			%let grouplist = "&analysisgrp";

		    /**********************/
		    /* Compute Numerators */
		    /**********************/

		    %macro createNumTableEntries(class=, level=);

		        *One record per patient for the class stratification;
		        proc means data=_mstr nway noprint missing;
		        var Patient AdjustedDisp RawDisp TotRxSup TotRxAmt HadEvent NumEvents tte;
		        class analysisgrp PatId &class.;
		        id patient;
		        output out=_num0(drop=_:) max(Patient)=Npts
		                                  sum(Patient)=Episodes           
		                                  sum(AdjustedDisp)=AdjustedCodeCount   
		                                  sum(RawDisp)=RawCodeCount   
		                                  sum(TotRxSup)=DaySupp
		    					          sum(TotRxAmt)=AmtSupp
		    						      sum(HadEvent)=Eps_wEvents
		                                  sum(NumEvents)=All_Events
		    							  sum(tte)=tte;             
		        run;

		        *One record per patient for the class stratification;
		        proc means data=_num0 missing nway noprint;
		        var Npts Episodes AdjustedCodeCount RawCodeCount DaySupp AmtSupp Eps_wEvents All_Events tte; ;
		        class analysisgrp &class.;
		        output out=_outlvl(drop=_:) sum=;             
		        run;
			
			        %if %eval(&s.=1) %then %do;
			            data _numcounts;
			            set _outlvl;
			            format level $3.;
			            Level="&level.";
			            run;
			        %end;
			        %else %do;
			            data _numcounts;
			            set  _numcounts
			                _outlvl(in=a);
			                if a then do;	
								Level="&level.";
							end;
			            run;
			        %end;

					proc datasets nowarn noprint lib=work;
						delete _outlvl _num0;
					quit;
		    %mend;

			%isdata(dataset=_t2conclevels);    
			%if %eval(&nobs.>0) %then %do;
				%do s = 1 %to %eval(&nobs.);
					data _null_;
						set _t2conclevels;
						if _n_ = &s. then do;
							call symputx('levelid', levelid);
							call symputx('levelvars', levelvars);
						end;
					run;

					*Square stratifications vars;
					%ms_squaredtableshell(squarevars =&levelvars.,
								  		  analysisvars=AdjustedDisp RawDisp TotRxSup TotRxAmt NumEvents tte,
								  		  groups=&grouplist.,
										  analysis=Y);

					/*Merge squared values with _mstr*/
					data _mstr;
						set dplocal.&runid._mstr_concomitance(in=a where=(analysisgrp in (&grouplist.)))
						    _mastersquare(in=b);
						if a then do;
							month=month(indexdt);
							patient=1;
						    if NumEvents>0 then do;
								HadEvent=1;
							end;
							else do;
								HadEvent=0;
								NumEvents = 0;
							end;
							if tte = . then tte = 0;
						end;
					    else do;
					        patient=0;    
					        HadEvent=0; 
					    end;
					run;
					
					/*Summarize data */
		   	 		%createNumTableEntries(class=&levelvars.,level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _mastersquare _mstr;
					quit;
				%end;
			%end;

			*The table to be generated in MSOC will be the result of a merge between numerators and donominators;
			data _numcounts;
				set _numcounts;
					*initialize missing vars to prevent merge w a r n i n g s;
					length race hispanic zip_uncertain $1. zip3 state hhs_reg cb_reg $7.;
						if _n_ = 1 then do;
						dsid = open("_NumCounts");
							if varnum(dsid,"race") = 0 then race ='';
							if varnum(dsid,"hispanic") = 0 then hispanic ='';
							if varnum(dsid,"zip_uncertain") = 0 then zip_uncertain ='';
							if varnum(dsid,"zip3") = 0 then zip3 ='';
							if varnum(dsid,"state") = 0 then state ='';
							if varnum(dsid,"hhs_reg") = 0 then hhs_reg ='';
							if varnum(dsid,"cb_reg") = 0 then cb_reg ='';

							if varnum(dsid,"sex") = 0 then sex ='';
							if varnum(dsid,"agegroup") = 0 then agegroup ='';
							if varnum(dsid,"agegroupnum") = 0 then agegroupnum =.;
							if varnum(dsid,"year") = 0 then year =.;
							if varnum(dsid,"month") = 0 then month =.;
						rc= close(dsid);
					end;
					drop rc dsid;
			run;

			proc sort data=_numcounts;
				by analysisgrp level sex AgeGroupNum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic  ;
			run; 

			*List of covariates;
			proc sql noprint;
				select distinct levelvars into: levelvars separated by ' '
				from _t&type._levels;
			quit;
			%nonrep(invar = levelvars, outvar = levelvars1);
			*put covariates into macro var;
			%do c = 1 %to %sysfunc(countw(&levelvars1.));
				%if %index(%scan(&levelvars1., &c.), covar) > 0 %then %do;
					%let covarstrat = &covarstrat. %scan(&levelvars1., &c.);
				%end;
			%end;

			* Output to DPLOCAL and MSOC;
			%if %eval(&group.=1) %then %do;
				data MSOC.&RUNID._t2_concomitance;
					retain analysisgrp Level sex agegroup agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic &covarstrat. Npts Episodes AdjustedCodeCount RawCodeCount DaySupp
				     		AmtSupp Eps_wEvents All_Events tte;
					set _numcounts;
					keep analysisgrp Level sex agegroup agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic &covarstrat. Npts Episodes AdjustedCodeCount RawCodeCount DaySupp
				     	AmtSupp Eps_wEvents All_Events tte;
					run;
			%end;
			%else %do;
				proc append base=MSOC.&RUNID._t2_concomitance
				            data=_numcounts(keep=analysisgrp Level sex agegroup agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain &covarstrat. race hispanic 
											Npts Episodes AdjustedCodeCount RawCodeCount DaySupp AmtSupp Eps_wEvents All_Events tte) force;
		     	run;
			%end;

		%end; *loop through groups;
	%end; *output table;

	%put NOTE: ******** END OF MACRO: ms_concomitantusetables v1.0 ********;

%mend ms_concomitantusetables;
