****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_switching.sas  
*
* Created (mm/dd/yyyy): 05/15/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Calculate switching episodes for Type 6 analysis                                     
*   
*  Program inputs:  
*	-infolder.&treatmentpathways
*	-infolder.&inclusioncodes
*	-dplocal.&runid._mstr
*	-msoc.&runid_t6_productsdates
*             	                        
*  Program outputs:    
*	-dplocal.&runid._T6_switchepisodes
*	-msoc.&runid._T6_switchattrition 
*	-msoc.&runid._T6_switchepisdurstats
*	-msoc.&runid._T6_switchplota
*	-msoc.&runid._T6_switchplotb 
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_switching();

	%put =====> MACRO CALLED: ms_switching v1.0; 

	*Determine which tables and strata to output;
	data _t6levels_switchepisdur
		 _t6levels_switchplota
		 _t6levels_switchplotb;
		set userstrata;
		
		*SwitchEpisDur;
		if lowcase(tableID) = "t6switchepisdur" then do;
			*defensive - add episodelength if not specified since that is an automatic strata;
			if index(lowcase(levelvars), 'episodelength') = 0 then do;
				levelvars = catx(' ',levelvars, "episodelength");
			end;
			output _t6levels_switchepisdur;
		end;

		*Plots a and b;
		if lowcase(tableID) = "t6plota" then do;
			*defensive - add TTSwitch if not specified since that is an automatic strata;
			if index(lowcase(levelvars), 'ttswitch') = 0 then do;
				levelvars = catx(' ',levelvars, "ttswitch");
			end;
			output _t6levels_switchplota;
		end;
		if lowcase(tableID) = "t6plotb" then do;
			*defensive - add TTSwitch if not specified since that is an automatic strata;
			if index(lowcase(levelvars), 'ttswitch') = 0 then do;
				levelvars = catx(' ',levelvars, "ttswitch");
			end;
			output _t6levels_switchplotb;
		end;
	run;

	%let t6switchepisdurstrat1=;
	%let t6plotastrat1=;
	%let t6plotbstrat1=;

	proc sql noprint;
		select distinct lowcase(levelvars) into: t6switchepisdurstrat1 separated by ' '
		from _t6levels_switchepisdur;
		select distinct lowcase(levelvars) into: t6plotastrat1 separated by ' '
		from _t6levels_switchplota;
		select distinct lowcase(levelvars) into: t6plotbstrat1 separated by ' '
		from _t6levels_switchplotb;
	quit;
	%nonrep(invar = t6switchepisdurstrat1, outvar = t6switchepisdurstrat);
	%nonrep(invar = t6plotastrat1, outvar = t6plotastrat);
	%nonrep(invar = t6plotbstrat1, outvar = t6plotbstrat);

	/***********************/
	/* Calculate Switching */
	/***********************/

	/*Read in treatmentpathways and loop through analysisgrp*/

	%ISDATA(dataset=infolder.&treatmentpathways.);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
	proc sql noprint;
		select distinct analysisgrp into: analysisgrp_list separated by ' '
		from infolder.&treatmentpathways.;
	quit;

	%let n_analysisgrp = %sysfunc(countw(&analysisgrp_list.));

	%do n = 1 %to %eval(&n_analysisgrp.);
		%let analysisgrp = %upcase(%scan(&analysisgrp_list., &n.));
		%put Now looping on analysisgrp = &analysisgrp.;

		data switch&n.;
			set infolder.&treatmentpathways.;
			where upcase(analysisgrp) = "&analysisgrp.";
		run;

			/*Defensive warnings*/
			proc sql noprint;
				select count(distinct switchcohortincldate),
					count(distinct switchdateuse)
					into: count_switchcohortincldate, :count_switchdateuse
				from switch&n.;
			quit;
			%if %eval(&count_switchcohortincldate. > 1) %then %do;
				%put WARNING: More than one row per unique analysisgrp contains a non-null SWITCHCOHORTINCLDATE value. Revise TREATMENTPATHWAYS file;
			%end;
			%if %eval(&count_switchdateuse. > 1) %then %do;
				%put WARNING: More than one row per unique analysisgrp contains a non-null SWITCHDATEUSE value. Revise TREATMENTPATHWAYS file;
			%end;

			/*Identify max SWITCHEVALSTEP*/
			proc sql noprint;
				select max(switchevalstep) into: max_switchevalstep
				from switch&n.;
			quit;
			%put &max_switchevalstep.;

		%do i = 0 %to %eval(&max_switchevalstep.);
			/* Assign macro parameters*/
			data _null_;
				set infolder.&treatmentpathways.;
				where upcase(analysisgrp) = "&analysisgrp." and switchevalstep = &i.;

				call symputx("switchgroup&i.", upcase(group));
				call symputx("switchcohortdef&i.", switchcohortdef);
				call symputx("gaptol&i.", gaptol);
				call symputx("overlaptol&i.", overlaptol);
				call symputx("overlaptype&i.", upcase(overlaptype));
				call symputx("switchcohortincldate&i.", upcase(switchcohortincldate));
				call symputx("switchdateuse&i.", switchdateuse);
				call symputx("switchgapincl&i.", upcase(switchgapincl));
			run;

			/*	Select all GROUP episodes for each SWITCHEVALSTEP. */
			/*  Note: All SWITCHEVALSTEP=0 episode will be considered as start of switch pattern, episodes that are a switch-back will be removed later */
			proc sort data=dplocal.&runid._mstr(rename=indexdt=indexdt&i. 
												rename=episodeenddt=episodeenddt&i. 
												rename=enr_end=enr_end&i. 
												rename=enr_start=enr_start&i.
												rename=cens_elig=cens_elig&i.
												rename=cens_dth=cens_dth&i.
												rename=cens_dpend=cens_dpend&i.
												rename=cens_qryend=cens_qryend&i.
												rename=cens_episend=cens_episend&i.
												rename=EpisodeEndDt_Censor=censor_date&i.
												rename=sex=sex&i.
												rename=agegroup=agegroup&i.
												rename=agegroupnum=agegroupnum&i.
												rename=race=race&i.
												rename=hispanic=hispanic&i.
												rename=zip3=zip3&i.
												rename=state=state&i.
												rename=hhs_reg=hhs_reg&i.
												rename=cb_reg=cb_reg&i.
												rename=zip_uncertain=zip_uncertain&i.)
				out=step&i._index(keep=patid indexdt&i. episodeenddt&i. enr_end&i. enr_start&i. cens: race&i. hispanic&i.
									sex&i. agegroup&i. agegroupnum&i. zip3&i. state&i. hhs_reg&i. cb_reg&i. zip_uncertain&i.); 
				by patid indexdt&i.; 
				where upcase(group) = "&&switchgroup&i.";
			run;
		%end;

		/*Create switching episodes*/
		%macro identifyswitch(step=);
			%let priorstep = %eval(&step.-1);

			**Use point= datastep option to select 1st SWITCHEVALSTEP 1 episode after SWITCHEVALSTEP 0 index date;
			data start_stop; 
				set step&step._index;
				by patid indexdt&step.; 
				retain start;
				if first.patid then start =_n_;
				if last.patid then stop = _n_;
				if last.patid;
				keep patid start stop;
			run;

			data _selectedindex(where=(indexdt&priorstep. ne .)); 
					length indexdt&priorstep. episodeenddt&priorstep. switchindexdt&step. switchepisodeenddt&step. 8;

				merge 	step&priorstep._index 
						start_stop; 
				by patid; 

				if (start ne .) and (stop ne .) then do recordid = start to stop;
					set step&step._index point = recordid;

					if switchindexdt&step. = . and indexdt&step. >= indexdt&priorstep. then do;
						switchindexdt&step. = indexdt&step.;
						switchepisodeenddt&step. = episodeenddt&step.;
					end;
				end;
				format indexdt: episodeenddt: switchindexdt: switchepisodeenddt:  date9.;
				drop start stop recordid indexdt&step. episodeenddt&step.;
			run;

		/*Determine if a switch occured*/
			data switchstep&step.;
				set _selectedindex;

				/*No switch occurs if:
					1. Switchindexdt&step. is missing (no step 1 episodes after step 0 index date) then switch_reason&priorstep. = No switch OR
					2. Switchindexdt&step. is > GAPTOL from episodeenddt&priorstep.*/

				if (switchindexdt&step. = .) OR 
						(switchindexdt&step. - episodeenddt&priorstep. -1 > %eval(&&gaptol&step.)) then do;
					switch&step. = "N";
				end;
				else do;
	
					/*Switch occurs if:
						1. overlap <= OVERLAPTOL OR
						2. Switchindexdt&step. <= GAPTOL from EpisodeEndDt&priorstep. */

					overlap = EpisodeEndDt&priorstep. - SwitchIndexDt&step. + 1;
					Percentoverlap = 100*(overlap/(EpisodeEndDt&priorstep.-IndexDt&priorstep. + 1));

					%if "&&overlaptype&step." = "PERCENT" %then %do;
						if Percentoverlap <= %eval(&&overlaptol&step.) then do;
							switch&step. = "Y";
						end;
						else if Percentoverlap > %eval(&&overlaptol&step.) then do;
							switch&step. = "N";
						end;
						else do;
					%end;
					%if "&&overlaptype&step." = "DAYS" %then %do;
						if overlap <= %eval(&&overlaptol&step.) then do;
							switch&step. = "Y";
						end;
						else if overlap > %eval(&&overlaptol&step.) then do;
							switch&step. = "N";
						end;
						else do;
					%end;

					if Switchindexdt&step. > EpisodeEndDt&priorstep. AND (SwitchIndexDt&step.-EpisodeEndDt&priorstep. -1) <= %eval(&&gaptol&step.) then do;
						switch&step. = "Y";
					end;
					end;
				end;

				/*Convert switches to non-switches if there is an enrollment gap in the gaptol*/
				if switch&step. = "Y" then do;
					if enr_start&step. > EpisodeEndDt&priorstep. then switch&step. = "N";
				end;

				if switch&step. ne "Y" then call missing(switchindexdt&step.,switchEpisodeEndDt&step., 
					cens_elig&step., cens_elig&step., cens_dth&step., cens_dpend&step., cens_qryend&step., cens_episend&step., censor_date&step.);

				/* Reclassify initial switch enddate to day prior to switch and keep original enddate for duration metrics  */
				/* Reclassification is to prevent inclusion of episodes that prior to SWITCHCOHORTINCLDATE + GAPTOL date    */
					format OrigEpisodeEndDt&priorstep. date9.;
					OrigEpisodeEndDt&priorstep. = EpisodeEndDt&priorstep.;
					if switch&step. = "Y" then EpisodeEndDt&priorstep. = switchindexdt&step.-1;
			run;
		%mend;
		%identifyswitch(step=1);

		%if %eval(&max_switchevalstep. = 2) %then %do;
			/*Evalute SWITCHEVALSTEP = 2 only for individuals who switched*/
			proc sort data=switchstep1(rename=switchindexdt1=indexdt1 rename=switchepisodeenddt1=episodeenddt1)
				out=step1_index(keep=patid indexdt1 episodeenddt1 enr_end1 enr_start1); 
				by patid indexdt1; 
				where switch1 = 'Y';
			run;
		
			%identifyswitch(step=2);

			/*Merge SWITCHEVALSTEP 1 and SWITCHEVALSTEP 2*/
			proc sql noprint;
				create table analysisgrp&n. as
				select one.patid
				,	one.indexdt0
				, 	one.episodeenddt0
				,	one.enr_end0
				,	one.enr_start0
				,	one.OrigEpisodeEndDt0
				,	one.switchindexdt1
				,	one.switchepisodeenddt1
				,	one.switch1
				,	one.cens_elig0
				,	one.cens_elig1
				,	one.cens_dth0
				,	one.cens_dth1
				,	one.cens_dpend0
				,	one.cens_dpend1
				,	one.cens_qryend0
				,	one.cens_qryend1
				,	one.cens_episend0
				,	one.cens_episend1
				,	one.censor_date0
				,	one.censor_date1
				,	two.switchindexdt2
				,	two.switchepisodeenddt2
				,	two.switch2
				,	one.sex0 as sex
				,	one.agegroup0 as agegroup
				,	one.agegroupnum0 as agegroupnum
				,	one.race0 as race
				,	one.hispanic0 as hispanic
				,	one.zip30 as zip3
				,	one.state0 as state
				,	one.hhs_reg0 as hhs_reg
				,	one.cb_reg0 as cb_reg
				,	one.zip_uncertain0 as zip_uncertain
				from switchstep1 as one
				left join switchstep2 as two 
				on one.switchindexdt1 = two.indexdt1 and one.patid=two.patid;
			quit;

			/*Remove switchback episodes as initial swithers*/
			%if "&switchgroup0." = "&switchgroup2." %then %do;
				proc sort data=analysisgrp&n.(keep=patid switchindexdt2) out=switchback;
				by patid switchindexdt2;
				where switchindexdt2 ne .;
				run;

				proc sort data=analysisgrp&n.; 
					by patid indexdt0;
				run;

				data analysisgrp&n.;
					merge analysisgrp&n. (in=a)
						switchback(rename=switchindexdt2=indexdt0 in=b);
						by patid indexdt0;
						if a and b then delete;
				run;
			%end;
		%end;
		%else %if %eval(&max_switchevalstep. = 1) %then %do;
			proc sql noprint;
				create table analysisgrp&n. as
				select one.patid
				,	one.indexdt0
				, 	one.episodeenddt0
				,	one.enr_end0
				,	one.enr_start0
				,	one.OrigEpisodeEndDt0
				,	one.switchindexdt1
				,	one.switchepisodeenddt1
				,	one.switch1
				,	one.cens_elig0
				,	one.cens_elig1
				,	one.cens_dth0
				,	one.cens_dth1
				,	one.cens_dpend0
				,	one.cens_dpend1
				,	one.cens_qryend0
				,	one.cens_qryend1
				,	one.cens_episend0
				,	one.cens_episend1
				,	one.censor_date0
				,	one.censor_date1
				,	one.sex0 as sex
				,	one.agegroup0 as agegroup
				,	one.agegroupnum0 as agegroupnum
				,	one.race0 as race
				,	one.hispanic0 as hispanic
				,	one.zip30 as zip3
				,	one.state0 as state
				,	one.hhs_reg0 as hhs_reg
				,	one.cb_reg0 as cb_reg
				,	one.zip_uncertain0 as zip_uncertain
				from switchstep1 as one;
			quit;
		%end;
			
		data analysisgrp&n.;
			set analysisgrp&n.;
			/*Calculte EndDate for Switch Episode*/
			switchpatternenddt = max(episodeenddt0, switchepisodeenddt1 %if %eval(&max_switchevalstep. = 2) %then %do;, switchepisodeenddt2 %end; );
			format switchpatternenddt date9.;
		run;

		/*Apply GAPTOL between switching episodes*/
		proc sort data=analysisgrp&n.;
			by patid indexdt0 switchpatternenddt;
	 	run;

		/*modified version of %ms_findgap()*/
		data analysisgrp&n.;
			set analysisgrp&n.;
			by patid;
			DaysUntreated=indexdt0-lag(switchpatternenddt);
			gap=DaysUntreated > max(&gaptol0.,0);
			if &gaptol0.=. then gap=0;  *If gaptol=., then only first date is valid;
			if first.patid then gap=1;  *first one always counts as being a potential index date;
			if gap or (&gaptol0.=0);  * if gaptol =0 then all claims are deemed potential indedate wrt own self;
			drop gap daysuntreated;
		run;

		%let INCDATEGROUP&n. = ;
		%let SWITCHCOHORTINCLDATE&n. = ;
		%let SWITCHDATEUSE&n. = ;
		%let gaptol&n. = ;
		%let switchenrdays&n. = ;
		%let switchenrdaysaftind&n. = ;
	
		/*Exclude switching episodes*/
			/* Apply SWITCHCOHORTINCLDATE, SWITCHDATEUSE, SWITCHENRDAYS, and SWITCHENRDAYSAFTIND */
				** Check to see if parameters have been specified;
				data _null_;
					set infolder.&treatmentpathways.;
					where upcase(analysisgrp) = "&analysisgrp." and switchcohortincldate is not missing;
						call symputx("INCDATEGROUP&n.", upcase(group));
						call symputx("SWITCHCOHORTINCLDATE&n.", upcase(SWITCHCOHORTINCLDATE));
						call symputx("SWITCHDATEUSE&n.", SWITCHDATEUSE);
						call symputx("gaptol&n.", gaptol);
				run;
				data _null_;
					set infolder.&treatmentpathways.;
					where upcase(analysisgrp) = "&analysisgrp." and switchevalstep = 0;
						call symputx("switchenrdays&n.", upcase(switchenrdays));
						call symputx("switchenrdaysaftind&n.", upcase(switchenrdaysaftind));
				run;

				%put &&INCDATEGROUP&n. &&SWITCHCOHORTINCLDATE&n. &&SWITCHDATEUSE&n. &&gaptol&n. &&switchenrdays&n. &&switchenrdaysaftind&n.;

				%if "&&SWITCHCOHORTINCLDATE&n." ne "" %then %do; /*if no SWITCHCOHORTINCLDATE, then do not apply inclusion criteria*/
					/*get date and evaluate whether SWITCHCOHORTINCLDATE + Gaptol occurs on or before EpisodeEndDt0*/
					proc sql noprint undo_policy=none;	
					create table analysisgrp&n. as
						select x.*,
						y.&&SWITCHCOHORTINCLDATE&n ,
						case when 
							x.episodeenddt0 + &&gaptol&n. >= y.&&SWITCHCOHORTINCLDATE&n then 0
							else 1
							end as excl_switchdate
						from analysisgrp&n. as x,
						msoc.&runid._t6_productsdates(where=(upcase(group)="&&INCDATEGROUP&n.")) as y;
					quit;
				%end;
				%else %do;
					data analysisgrp&n.;
						set analysisgrp&n.;
						excl_switchdate = 0;
					run;
				%end;

				*Evaluated enrollment requirements;
				data analysisgrp&n.;
					set analysisgrp&n.;

   					MinEnrolMet1=0; MinEnrolMet2=0; MinEnrolNotMet=0; * Initialize to prevent issues in switch_attrition;

					if enr_start0 <= indexdt0-max(0,&&switchenrdays&n.) then MinEnrolMet1=1;
					if enr_end0 >= indexdt0+max(0,&&switchenrdaysaftind&n.) then MinEnrolMet2=1;

					if MinEnrolMet1 ne 1 | MinEnrolMet2 ne 1 then MinEnrolNotMet = 1;
				run;


			/* Apply INCLUSIONCODES TO SWITCH EPISODE*/
				data switchinclusioncodes;
					set &cohortcodes.;
					where T6_INDEX in('INC','EXC') and upcase(group) = "&analysisgrp.";
				run;

				%isdata(dataset=switchinclusioncodes);
				    %IF %EVAL(&NOBS.>=1) %THEN %DO;

						*Processing steps for Inclusion/Exclusion codes;
						  data _diag 
						       _others;
						    set switchinclusioncodes;
						    if CodeCat in("DX") then output _diag;
						    else output _others;
						    run;

						    %ISDATA(dataset=_diag);
						    %IF %EVAL(&NOBS.>=1) %THEN %DO;
						        %MS_ProcessWildcards(InFile=_diag, 
						                             CodeVar=code, 
						                             OutFile=_diag);
						    %END;

						    *Create Switch Pattern-specific parameter files. These processing steps are the same as in ms_cidanum, however
								group=analysisgrp are only switchinclusion codes, which were not processed in ms_cidanum;
						    data _ITGroupParamNDC
						         _ITGroupParamMeds
						         _ITGroupParamLabs(rename=Group=RawGroup rename=CareSettingprincipal=RawCaresetting);
						       set _diag
						           _others;
						       if Condinclusion=. then Condinclusion=1;
						       if CodeCat="RX"          then output _ITGroupParamNDC;
						       if CodeCat in("DX","PX") then output _ITGroupParamMeds;
						       if CodeCat in("LB")      then output _ITGroupParamLabs;
						    run;

						    *Break out CaresettingPrincipal into Caresetting and Principal;
						    %ms_caresettingprincipal(InFile=_ITGroupParamMeds, 
						                             Var=CareSettingPrincipal, 
						                             OutFile=_ITGroupParamMeds);


							*Extract analysisgrp-specific records;
						    %ms_loopmeds(datafile=dplocal.meds,
						                 lookfile=_ITGroupParamMeds, 
						                 outfile=_ITMeds);

						    %ms_extractdrugs(datafile=dplocal.Drugs,
						                     datavar=NDC ,
						                     lookfile=_ITGroupParamNDC, 
						                     lookvar=code , 
						                     outfile=_ITDrugs);
	
							*Need to define Supply End Date to make sure days of supply are constrained within enrollment;
							data _ITDrugs;
							set _ITDrugs;
							format RxSupEndDt date9.;
							if not missing(codesupply) then RxSup = CodeSupply;
							RxSupEndDt=Rxdate+RxSup-1;
							run;

						    %ms_extractdrugs(datafile=dplocal.DrugsComb,
						                     datavar=NDC ,
						                     lookfile=_ITGroupParamNDC, 
						                     lookvar=code , 
						                     outfile=_ITDrugsComb);

							*Need to define Supply End Date to make sure days of supply are constrained within enrollment;
							data _ITDrugsComb;
							set _ITDrugsComb;
							format RxSupEndDt date9.;
							if not missing(codesupply) then RxSup = CodeSupply;
							RxSupEndDt=Rxdate+RxSup-1;
							run;

						    %ms_extractlabs(datafile=dplocal.labextract,
						                    lookfile=_ITGroupParamLabs, 
						                    outfile=_ITlabs);	

						    *Make sure all claims overlap enrollment span and only 
						     keep eligible exposures (overlaping Enr_Start Enr_End);
							  %ms_shaveoutside(reffile=analysisgrp&n.,
							                   refstart=Enr_Start0,
							                   refend=Enr_End0,
							                   KeepPartBf=N,
							                   ToShaveFile=_ITDrugs,
							                   ToShaveStart=RxDate,
							                   ToShaveEnd=RxSupEndDt,
							                   outfile=_ITDrugs);
							   
								%ISDATA(dataset=_ITDrugsComb);
								%IF %EVAL(&NOBS.>=1) %THEN %DO;
							        %ms_shaveoutside(reffile=analysisgrp&n.,
							                   		 refstart=Enr_Start0,
							                   		 refend=Enr_End0,
							                       	 KeepPartBf=N,
							                       	 ToShaveFile=_ITDrugsComb,
							                       	 ToShaveStart=RxDate,
							                       	 ToShaveEnd=RxSupEndDt,
							                       	 outfile=_ITDrugsComb(drop=Enr_Start0 Enr_end0));
							    %END;

							    %ms_shaveoutside(reffile=analysisgrp&n.,
							                   	 refstart=Enr_Start0,
							                   	 refend=Enr_End0,
							                     KeepPartBf=N,
							                     ToShaveFile=_ITMeds,
							                     ToShaveStart=ADate,
							                     ToShaveEnd=ADate,
							                     outfile=_ITMeds);
							   
							  	%ms_shaveoutside(reffile=analysisgrp&n.,
							                   	 refstart=Enr_Start0,
							                   	 refend=Enr_End0,
							                     KeepPartBf=N,
							                     ToShaveFile=_ITLabs,
							                     ToShaveStart=ADate,
							                     ToShaveEnd=ADate,
							                     outfile=_ITLabs);   

						*Apply Stockpiling;

						    *Default stockpiling parameters;
						    %let SAMEDAY=aa;
						    %let SUPRANGE=0<-HIGH;
						    %let AMTRANGE=0<-HIGH;
						    %let PERCENTDAYS=;

					    %ISDATA(dataset=infolder.&STOCKPILINGFILE.);	
					    %IF %EVAL(&NOBS.>0) %THEN %DO;
					        data &STOCKPILINGFILE.;
					        set infolder.&STOCKPILINGFILE.;
					        where upcase(group)="&analysisgrp.";
					        call symputx("SAMEDAY",strip(SameDay));
					        call symputx("SUPRANGE",strip(SupRange));
					        call symputx("AMTRANGE",strip(AmtRange));
					        call symputx("PERCENTDAYS",put(PercentDays,best.));
					        run;
					    %end;
         
						%let InclExclVars=CondInclusion CondFrom CondTo Cond Subcond SubCondInclusion codedays excludesupply;

					    %MS_STOCKPILING(INFILE=_ITDrugs,	  		
					                    CLMDATE=RxDate,                            		
					                    CLMSUP=RxSup,                              		
					                    CLMAMT=RxAmt,	                           		
					        		    PROCFLAG=,                                 		
					        		    PERCENTDAYS=&PERCENTDAYS.,                          		
					        		    GROUPING=Group StockGroup T1_INDEX T2_INDEX T2_FUP T3_INDEX T3_FUP T4_INDEX T4_FUP T5_INDEX T6_INDEX &InclExclVars.,                   		
					        		    SAMEDAY=&SAMEDAY.,                                 	
					        			SUPRANGE=&SUPRANGE.,									
					        			AMTRANGE=&AMTRANGE.,
					        		    ID=CODECAT,                                       		
					        		    OUTFILE=_ITDrugs,  		
					        			OUTFILEEXCL=_ITDrugsExcl
					                    );

					    data _ITDrugs;
						    set  _ITDrugs
					        	 _ITDrugsComb(in=a);
					    		*ExpireDt does not exist in _ITDrugsComb;
					   	 		if a then ExpireDt=Rxdate+RxSup-1; 	
					    run;

						*Need to reshave because Stockpiling may have pushed claims outside enrollment period;
					    %ISDATA(dataset=_ITDrugs);
						%IF %EVAL(&NOBS.>=1) %THEN %DO;
					        %ms_shaveoutside(reffile=analysisgrp&n.,
							                 refstart=Enr_Start0,
							                 refend=Enr_End0,
					                         KeepPartBf=N,
					                         ToShaveFile=_ITDrugs,
					                         ToShaveStart=RxDate,
					                         ToShaveEnd=ExpireDt,
					                         outfile=_ITDrugs);
					    %END;

					    proc datasets library=work nowarn nolist;
					    delete _ITGroupParam: _diag _others;
					    quit;


						data _InclExcl;
							set _ITDrugs(in=a rename=RxDate=ADate)
						        _ITMeds(in=b)
						        _ITLabs(in=c);

						        if b or c then do;
									if not missing(codesupply) then RxSup = CodeSupply;
						          	else RxSup = 1;
								    Expiredt=Adate+RxSup-1;
								    RXAmt=1;
									NumDispensing = 1;
						        end;
						  run;

						/*--------------------------------------------------------------*/
						/* Apply Exclusion/Inclusion Criteria (POV3)					*/
						/*--------------------------------------------------------------*/

				        *Are there any incl/excl conds to verify for this iteration?;
				        data _IT&inclusioncodes.;
				        set &inclusioncodes.;
				        where upcase(group)="&analysisgrp.";
				        run;

				        %ISDATA(dataset=_IT&inclusioncodes.);
				        %IF %EVAL(&NOBS.>=1) %THEN %DO;

				            *Initialize list with all potential index dates;
				            data _pov3;
				            set analysisgrp&n.;
				            MustKickOutHard=0;  *cumulative;
				            MustKickOutElig=0;  *cumulative;
				            keep PatId indexdt0 Enr_Start0 Enr_end0 MustKickOut:;
				            run;

				            *Uniqueness of _InclExcl;
				            Proc Sort nodupkey data=_InclExcl out=_InclExcl;
				            by Patid Adate Expiredt CondInclusion cond subcond SubCondInclusion CondFrom CondTo codedays excludesupply;
				            run;

				            *Get all in INCL/EXCL period claims (both incl and Excl) 
				             relative to each potential index dates;
				            proc sql noprint;
				            create table _pov3Claims(where=((ExcludeSupply='N' and WithSupply=1) | (ExcludeSupply='Y' and WithoutSupply=1))) as
				            select index.Patid,
				                   index.indexdt0,
				                   Inc.CondInclusion,
				                   Inc.cond,
				                   Inc.Subcond,
				                   Inc.SubCondinclusion,
								   Inc.excludesupply,
								   Inc.codedays,
								   %MS_PeriodsOverlap(period1=COALESCE(index.indexdt0+CondFrom,-999999) COALESCE(index.indexdt0+CondTo,999999),                                             
                                      period2=Inc.ADate Inc.Expiredt) as WithSupply ,
				  				   %MS_PeriodsOverlap(period1=COALESCE(index.indexdt0+CondFrom,-999999) COALESCE(index.indexdt0+CondTo,999999),                                             
                                      period2=Inc.ADate) as WithoutSupply
				            from analysisgrp&n. as index,
				                 _InclExcl as Inc
				            where index.Patid = Inc.Patid
				            order by index.Patid, 
				                     index.indexdt0;
				            quit;

							*Determine how many conditions;
							%let MAXCOND=;
							proc sql noprint;
							select max(cond) into: maxcond
							from _IT&inclusioncodes.;
							quit;

							data _NULL_;
							call symputx("MAXCOND",&MAXCOND.);*For sum issue;
							run;
							%put &MAXCOND.;


							%DO COND=1 %TO &MAXCOND.;

								%let MAXSUBCOND=0;
								*Determine how many levels of subconditions needed;
								proc sql noprint;
								select max(subcond) into: maxsubcond
								from _IT&inclusioncodes. (where=(cond=&COND.));
								quit;

								data _NULL_;
								call symputx("MAXSUBCOND",&MAXSUBCOND.);*For sum issue;
								run;
								%put &MAXSUBCOND.;

								*Determine whether this COND is an exclusion or inclusion;
								proc sql noprint;

									select max(CondInclusion) into: inclusion
									from _IT&inclusioncodes. (where=(cond=&COND.)); 

									select min(CondFrom) into: MinimumCondFrom
									from _IT&inclusioncodes. (where=(cond=&COND.)); 
								quit;
								%put &INCLUSION. &MinimumCondFrom.;

								*Set data for this cond;
								data _pov3_&cond.;
								set _pov3(keep=Patid indexdt0 ENR_START0 ENR_END0);
				                ExclCond&COND.=0;
								run;

								%DO SUBCOND=1 %TO &MAXSUBCOND.;

									%let subinclusion=0;
									*Determine if SUBCOND is an exclusion or inclusion;
									proc sql noprint;

									select max(SubCondInclusion) into: SUBCONDINCLUSION
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									select min(CondFrom) into: CondFrom
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									select max(codedays) into: codedays
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									quit;
									%put &SUBCONDINCLUSION. &CondFrom. &codedays.;


									*Only keep if codedays satisfied (number of ADates >= codedays);
									proc means data=_pov3Claims noprint nway missing;
										class patid Indexdt0 CondInclusion cond Subcond SubCondinclusion codedays / missing;
										output out=_pov3Incl_tmp(drop=_: where=(freq>=codedays)) n=freq;
										where cond=&COND. and subcond=&SUBCOND.;
									run;

							        proc sort nodupkey data=_pov3Incl_tmp out=_pov3Incl_tmp(keep=PatId Indexdt0);
							        	by PatId Indexdt0;  *Here Indexdt0 is of the analysisgrp, not the INCL;
							          	where cond=&COND. and subcond=&SUBCOND.;
							        run;

									*Square;						
									data _pov3_&cond.;
										merge _pov3_&cond.(in=a)
									      _pov3Incl_tmp(in=b);
										by PatId Indexdt0;

										if a and b then SubCond_&COND._&SUBCOND.=1;
										else SubCond_&COND._&SUBCOND.=0;

										if &SUBCONDINCLUSION.=1 then do;
											if a and b then SubCond_&COND._&SUBCOND.=1;
											else SubCond_&COND._&SUBCOND.=0;
										end;
										*If the subcondition is met but it is a subexclusion, then means that condition not satisfied;
										else do;
											if a and b then SubCond_&COND._&SUBCOND.=0;
											else SubCond_&COND._&SUBCOND.=1;
										end;
									run;

					  				*Check elig for the subconditions exclusions;
					          		*If the condition is a CondExclusion, then it will be checked again later with different assumptions;
										%if &SUBCONDINCLUSION.=0 and &inclusion.=1 %then %do;
											data _pov3_&cond.;
											set _pov3_&cond.;
												*check that the patient meets also meets elig for this subcondition;
							           		 if not(Enr_Start0 <= Indexdt0 + COALESCE(&CondFrom.,0) then do;*Open ended means -infty/infty;
							     				SubCond_&COND._&SUBCOND.=0;  
					                		 	ExclCond&COND.=1;
											end;
											run;
										%end;
								%END; *loop subcond;
								
								*If all subconditions are satisfied, then condition is satisfied (at this point);
								data _pov3_&cond.;
								set _pov3_&cond.;
								Cond&cond.=sum(of SubCond_&COND._1-SubCond_&COND._&MAXSUBCOND.)=&MAXSUBCOND.;
								run;
							
								%let condFrom2=.;
								data _null_;
								set _IT&inclusioncodes. (where=(cond=&COND.));
								if CondInclusion=0 then do;
									call symputx('CondFrom2',"&MinimumCondFrom.");
								end;
								run;
								%put &CondFrom. &CondFrom2.;

						        *Manage if an exclusion - if you have one hard exclusion 
						         or not enough elig, you are out;
								data _pov3_&cond.;
									set _pov3_&cond.;

									if &INCLUSION.=0 then do;
							          *if the patient had cond&cond.=1 then can never be included;
							          if Cond&cond.=1 then MustKickOutHard_&cond.=1;
							          else MustKickOutHard_&cond.=0;
							          *flip the dummy;
							          Cond&cond.=Cond&cond.=0;
							  			*Because we can have CondInclusion=0 and SubInclusion=1, we still need to consider if enough elig in this case;
							          MustKickOutElig_&cond.=0;
							          if not(Enr_Start0 <= Indexdt0 + COALESCE(&CondFrom2.,0)) then do;
							            MustKickOutElig_&cond.=1;
							            ExclCond&COND.=1;  *Open ended means -infty/infty;
							            Cond&cond.=0;
							          end;
									end;
									*Create a dummy to remember if this condition was an inclusion or and exlusion;
									InclCond&COND.=&INCLUSION.;
								run;

								*Put all together;
								data _pov3;
									merge _pov3(in=a)
									      _pov3_&cond.(in=b keep=PatId Indexdt0 Cond: SubCond: Excl: InclCond: MustKickOut:);
									by PatId Indexdt0;
							        if MustKickOutHard_&cond.=1 then MustKickOutHard=1;
							        if MustKickOutElig_&cond.=1 then MustKickOutElig=1;
									if a;
								run;
							%END;*loop cond;

							*Clean up;
							data _pov3;															
							set _pov3;
						      array _cond(*) Cond1-cond&maxcond.;
						      array _InclCond(*) InclCond1-InclCond&maxcond.;

						        *at least one inclusion criterion;
						        %if &maxcond. >1 %then %do;
									inclusion=max(of InclCond1-InclCond&maxcond.);
								%end;
								%else %do;
						        	inclusion=InclCond1;
								%end;

						        *initialize AnyCond;
						        if inclusion ne 1 then AnyCond=1;
						        else do;
						          AnyCond=0;
						          *Find those with at least on inclusion met;
						    			do i=1 to &maxcond.;
						            if _cond(i)=1 and _InclCond(i)=1 then AnyCond=1;
						    			end;	
						        end;
						 
						        *kick out any index not meeting the elig criteria for at least one exlcusion
						         or that has at least one hard exclusion (claim).;
						        if MustKickOutElig or MustKickOuthard then AnyCond=0;
						        drop inclusion;
						  run;

							data analysisgrp&n.;
								merge analysisgrp&n. (in=a)
								_pov3 (in=b keep=patid indexdt0 MustKickOutHard MustKickoutElig Anycond);
								by patid indexdt0;
								if a and b;
							run;
					%end; /*_ITINCLUSIONCODES*/
					%else %do;
						data analysisgrp&n.;
							set analysisgrp&n.;
								MustKickOutHard = 0;
								MustKickoutElig = 0;
								Anycond = 1;
						run;
					%end;
				%end;  /*INCLUSIONCODES*/
				%else %do;
					data analysisgrp&n.;
						set analysisgrp&n.;
							MustKickOutHard = 0;
							MustKickoutElig = 0;
							Anycond = 1;
					run;
				%end;

			/* Apply other criteria, then apply SWITCHCOHORTDEF */
				%if "&switchcohortdef0" = "01" %then %do; /*Only keep the 1st switching episode*/
					proc sort data=analysisgrp&n.;
						by patid indexdt0;
					run;

					data _temp_analysisgrp&n.;
						set analysisgrp&n.(where=(max(excl_switchdate, mustkickouthard, mustkickoutelig, MinEnrolNotMet)=0 and anycond = 1));
						by patid indexdt0;
						if first.patid then MetCohortdef = 1;
							else MetCohortdef = 0;
					run;

					/*merge back for attrition*/
					data analysisgrp&n.;
						merge analysisgrp&n.
							_temp_analysisgrp&n.(keep=patid indexdt0 MetCohortdef);
						by patid indexdt0;
					run;
				%end;
				%else %do;
					data analysisgrp&n.;
						set analysisgrp&n.;
						MetCohortdef = 1;
				%end;

		/*--------------------------------------*/
		/*---- Compute Metrics -----------------*/
		/*--------------------------------------*/

		data analysisgrp&n.;
			length analysisgrp $40;
			set analysisgrp&n.;

				/*compute time to 1st switch*/
				if switch1 = 'Y' then do;
					%if %eval(&&SWITCHDATEUSE&n. = 2) %then %do; /*Time FROM SWITCHCOHORTINCLDATE*/	
						time_to_first_switch = switchindexdt1 - &&SWITCHCOHORTINCLDATE&n.;
					%end;
					%else %do;
						time_to_first_switch = switchindexdt1 - indexdt0;
					%end;
						/*Compute time to 2nd switch (if applicable)*/
						%if %eval(&max_switchevalstep. = 2) %then %do;
							if switch2 = 'Y' then do; 
								time_to_second_switch = switchindexdt2 - switchindexdt1;
							end;
							else do;
								time_to_second_switch = censor_date1 - switchindexdt1;
							end;
						%end;
						%else %do;
							time_to_second_switch = .;
						%end;
				end; 
				else do;
					%if %eval(&&SWITCHDATEUSE&n. = 2) %then %do;
					time_to_first_switch = censor_date0 - &&SWITCHCOHORTINCLDATE&n.;
					%end;
					%else %do;
					time_to_first_switch = censor_date0 - indexdt0;
					%end;
					time_to_second_switch = .; /*No second switch if there is no 1st switch*/
				end;

				/*Compute switch pattern duration*/
				%if "&switchgapincl0." = "Y" %then %do; /*include gaps*/
					episodelength = switchpatternenddt - indexdt0 + 1;
				%end;
				%else %do; /*do not include gaps*/
					/*Gap between SwitchEval step 1 and 0*/
					gap0 = switchindexdt1 - Origepisodeenddt0 - 1; /*minus 1 b/c a 1 day gap means EpisodeEndDt and SwitchIndexDt separated by 2 days*/

					/*Gap between SwitchEval step 2 and 1*/
					%if %eval(&max_switchevalstep. = 2) %then %do;
						gap1 = switchindexdt2 - switchepisodeenddt1 - 1;
					%end;
					%else %do;
						gap1 = .;
					%end;
					episodelength = (switchpatternenddt - indexdt0 + 1) - (max(gap0,0)) - (max(gap1,0));
				%end;

			analysisgrp = "&analysisgrp.";
			all=1;
			year=year(indexdt0);

			/*Reclassify censoring if a switch*/
			%do a = 1 %to &max_switchevalstep.;
				if switch&a. = "Y" then do;
					cens_dth%eval(&a.-1) = 0; 
					cens_qryend%eval(&a.-1) = 0;
					cens_dpend%eval(&a.-1) = 0;
					cens_elig%eval(&a.-1) = 0;
					cens_episend%eval(&a.-1) = 0;
					cens_switch%eval(&a.-1) = 1;
				end;
				else do;
					cens_switch%eval(&a.-1) = 0;
				end;
			%end;
		run;

		/*************************/
		/* T6_SWITCHATTRITION	 */ 
		/*************************/	

		*Reset cumulative;
		    proc datasets library=work nowarn nolist;
			    delete _attrition;
		    quit;

			%let level=1;

			/** Utility macro %switch_attrition a modified version of %attrition from ms_attrition	**/ 
			/**	to select episodes instead of patients												**/
				 %macro switch_attrition(file=,ToExcl=);

				    proc sql noprint;
				    select count(*) into: NumToExcl
				    from &file.
				    where not (&ToExcl.);
				    quit;
				    %put &NumToExcl.;
				    data &file.;
				    set &file.;
				    where not (&ToExcl.);
				    run;

				    %ISDATA(dataset=_Attrition);
				    %IF %EVAL(&NOBS.>=1) %THEN %DO;
				        data _Attrition;
				        set _Attrition end=eof;
				        output;
				        if eof then do;
				            Level=input("&level.",best.);
				            Num=input("&NumToExcl.",best.);
				            call symputx("level",&level.+1);
				            output;
				        end;
				        run;
				    %END;
				    %ELSE %DO;
				        data _Attrition;
				        format level best. num comma10.;
				        Level=input("&level.",best.);
				        Num=input("&NumToExcl.",best10.);
				        call symputx("level",&level.+1);
				        output;
				        run;
				    %END;
				    %put %eval(&level.-1) &NumToExcl.;
				 %mend; 

			    %switch_attrition(file=analysisgrp&n.,ToExcl=all=0);
 				%switch_attrition(file=analysisgrp&n.,ToExcl=excl_switchdate=1);
 				%switch_attrition(file=analysisgrp&n.,ToExcl=MinEnrolNotMet=1);
		        %switch_attrition(file=analysisgrp&n.,ToExcl=MustKickOutElig=1);
		        %switch_attrition(file=analysisgrp&n.,ToExcl=MustKickOutHard=1);
		        %switch_attrition(file=analysisgrp&n.,ToExcl=AnyCond=0);
       			%switch_attrition(file=analysisgrp&n.,ToExcl=MetCohortdef=0);

				    data _Attrition;
					    retain analysisgrp level descr num Excluded;
					    format analysisgrp $40. descr $500.;
					    set _Attrition;
					    analysisgrp="&analysisgrp.";

					    format ;
					    LastRemain=lag(num);
					    Excluded=LastRemain-num;

						if level=1 then descr="Initial Switch Episode Count";
						if level=2 then descr="Exclusion - Episodes must satisfy switch cohort entry criteria (SWITCHCOHORTINCLDATE and SWITCHDATEUSE)";
				       	if level=3 then descr="Exclusion - Episodes must satisfy the switch enrollment requirement";
				       	if level=4 then descr="Exclusion - Episodes must satisfy the Exclusion enrollment requirement";
				        if level=5 then descr="Exclusion - Episodes must satisfy the Exclusion conditions";
				        if level=6 then descr="Exclusion - Episodes must satisfy the Inclusion conditions";
				        if level=7 then descr="Exclusion - Episodes must meet SWITCHCOHORTDEF requirement";

					    rename num=remaining;
					    drop LastRemain;
				    run;

			%if %eval(&n. = 1) %then %do;
				/*Keep final episodes*/
				data dplocal.&runid._t6_switchepisodes;
					set analysisgrp&n.(where=(max(excl_switchdate, mustkickouthard, mustkickoutelig)=0 and anycond = 1 and MetCohortdef=1));
				run;

				/*Output attrition table to MSOC folder*/
				data MSOC.&runid._t6_switchattrition;
					retain analysisgrp level descr remaining Excluded;
				    set _Attrition;
				run;
			%end;
			%else %do;
				data dplocal.&runid._t6_switchepisodes;
					set dplocal.&runid._t6_switchepisodes
						analysisgrp&n.(where=(max(excl_switchdate, mustkickouthard, mustkickoutelig)=0 and anycond = 1 and MetCohortdef=1));
				run;

		        proc append base=MSOC.&runid._t6_switchattrition 
		                    data=_Attrition force;
		        run;
			%end;

		proc datasets nowarn noprint;
			delete step: start_stop ;
		quit;

		/*--------------------------------------*/
		/*---- Output switching tables ---------*/
		/*--------------------------------------*/

		data _null_;
			set infolder.&cohortfile.;
			where upcase(cohortgrp) = "&switchgroup0";
			if missing(agestrat) then do;
				call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
			end;
			else do;
				call symputx("agestrat",agestrat);
			end;	
		run;

	  	proc sql noprint;
		    create table _switchers&n. as 
		    select mstr.*,
				   month(indexdt0) as month
		    from analysisgrp&n.(where=(max(excl_switchdate, mustkickouthard, mustkickoutelig)=0 and anycond = 1 and MetCohortdef=1)) as mstr;
		quit;	

		/******************************************************/
		/*  T6_SWITCHPLOTA and T6_SWITCHPLOTB (if applicable) */
		/******************************************************/
		%macro createTTS(class=,level=);

	        *One record per patient for the class stratification;
	        proc means data=_switchers_tts nway noprint missing;
	        var cens_elig&num. cens_dth&num. cens_dpend&num. cens_qryend&num. cens_episend&num. cens_switch&num.;
	        class analysisgrp PatId &class.;
	        output out=_num0(drop=_:) 	max(cens_elig&num.)=EndEnrollmentPatCount           
	                                  	max(cens_dth&num.)=DeathPatCount  
										max(cens_dpend&num.)=EndAvailDataPatCount          
										max(cens_qryend&num.)=EndQueryPatCount 
										max(cens_episend&num.)=ProductDiscontinuationPatCount                 
										max(cens_switch&num.)=SwitchedPatCount    
	                                  	sum(cens_elig&num.)=EndEnrollmentCount           
	                                  	sum(cens_dth&num.)=DeathCount  
										sum(cens_dpend&num.)=EndAvailDataCount           
										sum(cens_qryend&num.)=EndQueryCount 
										sum(cens_episend&num.)=ProductDiscontinuationCount    
										sum(cens_switch&num.)=SwitchedCount;        
	        run;

	        *One record per patient for the class stratification;
	        proc means data=_num0 missing nway noprint;
	        var EndEnrollmentPatCount DeathPatCount EndAvailDataPatCount EndQueryPatCount ProductDiscontinuationPatCount SwitchedPatCount
				EndEnrollmentCount DeathCount EndAvailDataCount EndQueryCount ProductDiscontinuationCount SwitchedCount;
	        class analysisgrp &class.;
	        output out=_tts_strat(drop=_:) sum=;             
	        run;

			%if %eval(&s.=1) %then %do;
		    	data _TTS;
		            set _tts_strat;
		            format level $3.;
		            Level="&level.";
		        run;
			%end;
		    %else %do;
		    	data _TTS;
		            set _TTS
		                _tts_strat(in=a);
		            if a then Level="&level.";
		        run;
			%end;
		%mend;

		%macro time_to_switch_plot(num=, var=, plot=);

			%isdata(dataset=_t6levels_switchplot&plot.);    
			%if %eval(&nobs.>0) %then %do;

				/*Square time_to_switch to all values*/
				proc sql noprint;
					create table _TTS_squared as
					select distinct &var. 
					from _switchers&n.
					where &var. is not missing;
				quit;

				%isdata(dataset=_TTS_squared)
				%if %eval(&nobs.=0) %then %do;
					%put WARNING: No qualifying switch episodes for &analysisgrp.;
				%end;

				%isdata(dataset=_t6levels_switchplot&plot.);    
				%do s = 1 %to %eval(&nobs.);
					data _null_;
						set _t6levels_switchplot&plot.;
						if _n_ = &s. then do;
							call symputx('levelid', levelid);
							call symputx('levelvars', levelvars);
						end;
					run;

					*Square stratifications vars;
					%ms_squaredtableshell(squarevars =&levelvars.,
								  		  analysisvars=EndEnrollmentCount DeathCount EndAvailDataCount EndQueryCount EndProductDiscontinuationCount 
													   switchedcount SwitchedPatCount EndQueryPatCount EndEnrollmentPatCount EndAvailDataPatCount 
													   ProductDiscontinuationPatCount deathpatcount,
								  		  groups="&analysisgrp",
										  analysis = Y);					
					
					proc sort data=_mastersquare nodup;
						by _all_;
					run;

					data _switchers_tts (rename=&var.=ttswitch);
					    set _switchers&n.(in=a where=(&var. is not missing))
					        _mastersquare(in=b);
					    if a then do;
							patient=1;
					  	end;
						else do;
							patient=0;
							cens_elig&num. = 0;
							cens_dth&num. = 0;
							cens_dpend&num. = 0;
							cens_qryend&num. = 0;
							cens_episend&num. = 0;
							cens_switch&num. = 0;
						end;
					run;

					*Summarize;
					%createTTS(class=&levelvars., level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _tts_strat _switchers_tts _mastersquare;
					quit;
				%end;

				/*Save to MSOC*/
				%if %eval(&n.=1) %then %do;
					/*Save to MSOC*/
					data msoc.&runid._t6_switchplot&plot.; 
						retain analysisgrp level &t6plotastrat. ttswitch 
						EndEnrollmentPatCount DeathPatCount EndAvailDataPatCount EndQueryPatCount ProductDiscontinuationPatCount SwitchedPatCount
						EndEnrollmentCount DeathCount EndAvailDataCount EndQueryCount ProductDiscontinuationCount SwitchedCount;
						set _TTS;
					run;
				%end;
				%else %do;
					proc append base=msoc.&runid._t6_switchplot&plot.
						data=_TTS(keep=analysisgrp level &t6plotastrat. ttswitch 
						EndEnrollmentPatCount DeathPatCount EndAvailDataPatCount EndQueryPatCount ProductDiscontinuationPatCount SwitchedPatCount
						EndEnrollmentCount DeathCount EndAvailDataCount EndQueryCount ProductDiscontinuationCount SwitchedCount) force;
					run;
				%end;

				proc datasets nowarn noprint lib=work;
					delete _tts;
				quit;
			%end;
		%mend;

		%time_to_switch_plot(num=0, var=time_to_first_switch, plot=a);

		proc sql noprint;
			select max(switchevalstep) into: max_switchevalstep
			from infolder.&treatmentpathways
			where upcase(analysisgrp) = "&analysisgrp";
		quit;
		%put &max_switchevalstep.;
		%if %eval(&max_switchevalstep. = 2) %then %do;
		%time_to_switch_plot(num=1, var=time_to_second_switch, plot=b);
		%end;

		/*************************/
		/* T6_SWITCHEPISDURSTATS */ 
		/*************************/	
		
		/*Square episodelength to all values*/
		proc sql noprint;
			create table _length_squared as
			select distinct episodelength
			from _switchers&n.;
		quit;
		
		%macro createduration(class=,level=);
			*One record per patient for the class stratification;
		    proc means data=_switchers_duration&n. nway noprint missing;
		        var patient;
		        class analysisgrp PatId &class.;
		        output out=_num0(drop=_:) max(patient)=Count;        
		    run;

		    *One record per patient for the class stratification;
		    proc means data=_num0 missing nway noprint;
		        var Count;
		        class analysisgrp &class.;
		        output out=_num0out(drop=_:) sum=;             
		    run;

			%if %eval(&s.) = 1 %then %do;
			   	data _Duration;
				    set _num0out;
			        format level $3.;
			        Level="&level.";
			    run;
			%end;
			%else %do;
			   	data _Duration;
				    set _Duration
			            _num0out(in=a);
			        if a then Level="&level.";
			     run;
			%end;
		%mend;


		%isdata(dataset=_t6levels_switchepisdur);    
		%if %eval(&nobs.>0) %then %do;
		%do s = 1 %to %eval(&nobs.);
			data _null_;
				set _t6levels_switchepisdur;
				if _n_ = &s. then do;
					call symputx('levelid', levelid);
					call symputx('levelvars', levelvars);
				end;
			run;

			*Square stratifications vars;
			%ms_squaredtableshell(squarevars =&levelvars.,
								  analysisvars=,
								  groups="&analysisgrp",
								  analysis = Y);
		
			proc sort data=_mastersquare nodup;
				by _all_;
			run;

		  	data _switchers_duration&n.;
			    set _switchers&n.(in=a)
			        _mastersquare(in=b);
			    if a then do;
					patient=1;
			  	end;
				else do;
					patient=0;
				end;
			run;

			%createduration(class=&levelvars. ,level=&levelid.);

			proc datasets nowarn noprint lib=work;
				delete _num: _switchers_duration _mastersquare;
			quit;
		%end;

		/*Save to MSOC*/
		%if %eval(&n.=1) %then %do;
			data msoc.&runid._t6_switchepisdurstats; 
				retain analysisgrp level &t6switchepisdurstrat. episodelength count;
				set _Duration;
			run;
		%end;
		%else %do;
			proc append base=msoc.&runid._t6_switchepisdurstats
			 			data=_Duration(keep=analysisgrp level &t6switchepisdurstrat. episodelength count) force;
			run;
		%end;
		%end;

		proc datasets nowarn noprint;
			delete _switch:;
		quit;

	%end; /*Loop through analysisgrp*/
	%end;

	proc datasets nowarn noprint;
		delete switch: ;
	quit;

%put NOTE: ********END OF MACRO: ms_switching v1.0********;

%mend;
